﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Text.RegularExpressions;

namespace RenrenImportProcess
{
    public static class ContactDetailMgr
    {

        /// <summary>
        /// 下载名片基本信息
        /// </summary>
        /// <param name="cookies"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        public static string GetDetail(CookieContainer cookies, string id)
        {
            return new ContactDetailDownloader(cookies, id).Download();
        }


        /// <summary>
        /// 下载个人主页通讯资料信息
        /// </summary>
        /// <param name="cookies"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        public static string GetPersonalContectsInfo(CookieContainer cookies, string id)
        {
            return new ContactDetailDownloader(cookies, id).DownlaodPersonalContactsInfo();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cookies"></param>
        /// <returns></returns>
        public static string SendAddMeAsFriend(CookieContainer cookies, string currentUserId, string message = "")
        {
            return new ContactDetailDownloader(cookies).SendAddMeRequest(message, currentUserId);
        }

        /// <summary>
        /// 下载个人头像
        /// </summary>
        /// <param name="cookies"></param>
        /// <param name="id"></param>
        /// <param name="url"></param>
        /// <param name="savingPath"></param>
        /// <returns></returns>
        public static string GetPhoto(CookieContainer cookies, string id, string url, string savingPath)
        {
            return new ContactDetailDownloader(cookies, id).DownloadDetailPhoto(url, savingPath);
            //WebClient wc = new WebClient();
            //wc.DownloadFile(url, savingPath);
        }

        /// <summary>
        /// 下载个人主页的详细信息
        /// </summary>
        /// <param name="cookie"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        public static string GetDetailedPersonalInfo(CookieContainer cookie, string id)
        {
            return new ContactDetailDownloader(cookie, id).GetPersonalDetailedInfo();
        }

    }

    public class ContactDetailDownloader
    {
        public readonly string CONTACTSFETCHURL = @"http://www.renren.com/showcard?friendID={0}";
        public readonly string PROFILECONTACT = @"http://friend.renren.com/getprofilecontact/{0}";
        public readonly string DETAILEDPERSONALINFO = @"http://www.renren.com/profile.do?id={0}&v=info_ajax&undefined";
        public readonly string ADDMEASFRIEND = @"http://friend.renren.com/ajax_request_friend.do?from=sg_findfriend_peopleumayknow&";
        private CookieContainer cookies;
        private string id;
        private string contactDeatils;

        public string ContactDeatils
        {
            get { return contactDeatils; }
            private set { contactDeatils = value; }
        }

        public ContactDetailDownloader(CookieContainer cookies, string id)
        {
            // TODO: Complete member initialization
            this.cookies = cookies;
            this.id = id;
        }

        public ContactDetailDownloader(CookieContainer cookies)
        {
            // TODO: Complete member initialization
            this.cookies = cookies;
        }

        public string Download()
        {
            HttpWebRequest web = (HttpWebRequest)WebRequest.Create(
                                string.Format(CONTACTSFETCHURL, id)
                                );
            web.CookieContainer = cookies;
            web.Method = "GET";
            web.Referer = "http://www.renren.com/ajaxproxy.htm";
            //web.ContentType = "application/x-www-form-urlencoded";
            web.AllowAutoRedirect = true;
            var response = web.GetResponse();
            string responseText;
            using (var receiveStream = response.GetResponseStream())
            {
                using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
                {
                    responseText = readStream.ReadToEnd();
                    readStream.Close();
                }
                receiveStream.Close();
            }
            response.Close();
            return responseText;
        }

        public string DownlaodPersonalContactsInfo()
        {
            HttpWebRequest web = (HttpWebRequest)WebRequest.Create(
                    string.Format(PROFILECONTACT, id)
                    );
            web.CookieContainer = cookies;
            web.Timeout = 18000;
            web.Method = "GET";
            web.Referer = "http://www.renren.com/ajaxproxy.htm";
            //web.Referer = "http://www.renren.com/profile.do?id=" + id + "&from=myFoot_goToVisit_nameToPrf";
            //web.ContentType = "application/x-www-form-urlencoded";
            web.AllowAutoRedirect = true;
            var response = web.GetResponse();
            string responseText;
            using (var receiveStream = response.GetResponseStream())
            {
                using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
                {
                    responseText = readStream.ReadToEnd();
                    readStream.Close();
                }
                receiveStream.Close();
            }
            response.Close();
            return responseText.Replace("http://", string.Empty);
        }

        public string GetPersonalDetailedInfo()
        {
            HttpWebRequest web = (HttpWebRequest)WebRequest.Create(
                string.Format(DETAILEDPERSONALINFO, id)
            );
            web.CookieContainer = cookies;
            web.Timeout = 18000;
            web.Method = "GET";
            web.Referer = "http://www.renren.com/ajaxproxy.htm";
            //web.ContentType = "application/x-www-form-urlencoded";
            web.AllowAutoRedirect = true;
            var response = web.GetResponse();
            string responseText;
            using (var receiveStream = response.GetResponseStream())
            {
                using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
                {
                    responseText = readStream.ReadToEnd();
                    readStream.Close();
                }
                receiveStream.Close();
            }
            response.Close();
            return responseText;
        }

        public string DownloadDetailPhoto(string url, string savingPath)
        {
            //savingPath = ReplaceBadCharOfFileName(savingPath);
            HttpWebRequest web2 = (HttpWebRequest)WebRequest.Create(url);
            web2.Timeout = 14000;
            web2.CookieContainer = cookies;
            web2.Method = "GET";
            web2.Referer = "http://friend.renren.com/myfriendlistx.do";
            //web.ContentType = "application/x-www-form-urlencoded";
            web2.AllowAutoRedirect = true;
            using (var response2 = web2.GetResponse())
            {
                //var fileBin = new byte[response2.ContentLength+1];
                using (MemoryStream ms = new MemoryStream())
                {
                    using (var receiveStream = response2.GetResponseStream())
                    {
                        var buff = new byte[64];
                        int c = 0; //实际读取的字节数
                        int total = 0;//total
                        //receiveStream.Read(buff, 0, Convert.ToInt32(response.ContentLength));
                        using (FileStream fs = new FileStream(savingPath, System.IO.FileMode.CreateNew))
                        {
                            using (BinaryWriter w = new BinaryWriter(fs))
                            {
                                //c = receiveStream.Read(fileBin, 0, fileBin.Length);
                                //w.Write(fileBin, 0, c);
                                while ((c = receiveStream.Read(buff, 0, buff.Length)) > 0)
                                {
                                    w.Write(buff, 0, c);
                                    ms.Write(buff, 0, c);
                                    total += c;
                                }
                                w.Close();
                            }
                            fs.Close();
                        }
                        //using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
                        //    responseText = readStream.ReadToEnd();
                        receiveStream.Close();
                    }
                    var retval = EncodeBase64(ms.ToArray());
                    ms.Close();
                    response2.Close();
                    return retval;
                }
            }
        }



        public string SendAddMeRequest(string message, string currentUserId)
        {
            /*
             * <input type="hidden" name="requestToken" value="-1217040841">
             * <input type="hidden" name="_rtk" value="83366c91">
             * */


            /*
             * XN = {get_check:'-1217040841',get_check_x:'83366c91',
             * */
            string requestToken = string.Empty;
            string _rtk = string.Empty;
            var regexRequestToken = new Regex(@"XN = {get_check:'(?<requestToken>[\s\S]*?)',get_check_x:'(?<_rtk>[\s\S]*?)',");
            //var regex_rtk = new Regex(@"<input type=""hidden"" name=""_rtk"" value=""(?<_rtk>[\s\S]*?)"">");

         

            try
            {
                HttpWebRequest web = (HttpWebRequest)WebRequest.Create("http://www.renren.com");
                web.CookieContainer = cookies;
                web.Timeout = 18000;
                web.Method = "GET";
                //web.Referer = "http://www.renren.com/";
                //web.ContentType = "application/x-www-form-urlencoded";
                web.AllowAutoRedirect = true;
                var response = web.GetResponse();
                string responseText;
                using (var receiveStream = response.GetResponseStream())
                {
                    using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
                    {
                        responseText = readStream.ReadToEnd();
                        readStream.Close();
                    }
                    receiveStream.Close();
                }
                response.Close();

                //Get the requesttoken and the _rtk
                var matchRequestToken = regexRequestToken.Match(responseText);
                if (matchRequestToken != null && matchRequestToken.Length == 0)
                {
                    //no request token has been found!
                    new RenrenExceptions("No request token! response text:" + responseText);
                    return null;
                }
          
                requestToken = matchRequestToken.Groups["requestToken"].Value;
                _rtk = matchRequestToken.Groups["_rtk"].Value;
                if (string.IsNullOrEmpty(requestToken) || string.IsNullOrEmpty(_rtk))
                {
                    //can not found the _rtk or requesttoke value in regex
                    new RenrenExceptions("Regex execute failed!");
                    return null;
                }
                new RenrenExceptions("REQUEST TOKEN:" + requestToken + " _rtk=" + _rtk);
            }
            catch (Exception ex)
            {
                new RenrenExceptions("Add Friend First Step Error", ex);
                return string.Empty;
            }

            /* 2012.1.20
             * http://friend.renren.com/ajax_request_friend.do?from=sg_findfriend_peopleumayknow&
             * */

            /* 2012.1.19
             * http://friend.renren.com/ajax_request_friend.do?from=opensearch&id=285934481&q=%E8%94%A1%E8%99%B9&p=&s=&u=246015810&act=addF&rt=user&in=9&ft=&hh=1& 
             * */
            //id=200128369&why=~~&codeFlag=0&code=&requestToken=-1217040841&_rtk=83366c91
            //id=240861885&why=Hello。&codeFlag=0&code=&requestToken=
            //http://friend.renren.com/ajax_request_friend.do?from=sg_findfriend_peopleumayknow&id=XXXXXXXXX&why=Hello。&codeFlag=0&code=&requestToken=

            try
            {
                HttpWebRequest web = (HttpWebRequest)WebRequest.Create(ADDMEASFRIEND);
                web.Headers["Origin"] = "http://friend.renren.com";
                string postdata = string.Format("id=246015810&why={0}&codeFlag=0&code=&requestToken={1}&_rtk={2}", message, requestToken, _rtk);
                new RenrenExceptions("DEBUG INFO ADD FRIEND POSTDATA:" + postdata); 
                web.CookieContainer = cookies;
                web.Timeout = 18000;
                web.Method = "POST";
                web.ContentLength = postdata.Length;
                web.Referer = "http://friend.renren.com/ajaxproxy.htm";
                web.ContentType = "application/x-www-form-urlencoded";
                web.AllowAutoRedirect = true;
                web.UserAgent = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E)";
                string responseText = string.Empty;
                using (Stream newStream = web.GetRequestStream())
                {
                    byte[] postBytes = Encoding.ASCII.GetBytes(postdata);
                    newStream.Write(postBytes, 0, postBytes.Length);
                    var responseData = (HttpWebResponse)web.GetResponse();
                    using (var stream = responseData.GetResponseStream())
                    {
                        var sr = new StreamReader(stream, Encoding.UTF8);
                        responseText = sr.ReadToEnd();
                        web.Abort();
                        new RenrenExceptions("DEBUG INFO ADD FRIEND:" + responseText);
                        return responseText;
                    }
                }
            }
            catch (Exception ex)
            {
                new RenrenExceptions("Add Friend Second Step Error", ex);
                return string.Empty;
            }
            //var response = web.GetResponse();

            //string responseText;
            //using (var receiveStream = response.GetResponseStream())
            //{
            //    using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
            //    {
            //        responseText = readStream.ReadToEnd();
            //        readStream.Close();
            //    }
            //    receiveStream.Close();
            //}
            //response.Close();
            //return responseText;
        }


        public string EncodeBase64(byte[] bytes)
        {
            string encode = "";
            try
            {
                encode = Convert.ToBase64String(bytes);
            }
            catch
            {
                encode = string.Empty;
            }
            return encode;
        }

        public string DecodeBase64(string code_type, string code)
        {
            string decode = "";
            byte[] bytes = Convert.FromBase64String(code);
            try
            {
                decode = Encoding.GetEncoding(code_type).GetString(bytes);
            }
            catch
            {
                decode = code;
            }
            return decode;
        }


        /// <summary>
        /// 去掉文件名中的无效字符,如 \ / : * ? " < > | 
        /// </summary>
        /// <param name="fileName">待处理的文件名</param>
        /// <returns>处理后的文件名</returns>
        public static string ReplaceBadCharOfFileName(string fileName)
        {
            string str = fileName;
            str = str.Replace("\\", string.Empty);
            str = str.Replace("/", string.Empty);
            str = str.Replace(":", string.Empty);
            str = str.Replace("*", string.Empty);
            str = str.Replace("?", string.Empty);
            str = str.Replace("\"", string.Empty);
            str = str.Replace("<", string.Empty);
            str = str.Replace(">", string.Empty);
            str = str.Replace("|", string.Empty);
            str = str.Replace(" ", string.Empty);
            str = str.Replace(".", string.Empty);
            str = str.Replace("-", string.Empty);
            return str;
        }
    }
}
